#ifndef TDMONITOR_H
#define TDMONITOR_H

#include <QtCore/QTimer>
#include <QtCore/QMutex>
#include <memory>
#include <deque>

#include "ui_TDMonitor.h"
#include "PP100API.h"

#define	TCHAR		char
#define	MAX_PATH	(260)
#define _T(x)		x

#if !defined(_countof)
template <typename Type, size_t Size>
char (*__countof_helper(Type (&ar)[Size]))[Size];
#define _countof(ar) sizeof(*__countof_helper(ar))
#endif//!defined(_countof)

const char g_pPP100APIDLL[] = "/opt/epson/Total Disc Maker/lib/libPP100API.so";

class PP100API_POINTER
{
private:
	void * hPP100APIDLL = NULL;
	std::string m_dlerror;

public:
	unsigned long ulHandle = 0;
	PP100_INITIALIZE         PP100_Initialize = NULL;
	PP100_DESTROY            PP100_Destroy = NULL;
	PP100_CONNECTSERVER      PP100_ConnectServer = NULL;
	PP100_DISCONNECTSERVER   PP100_DisConnectServer = NULL;
	PP100_ENUMPUBLISHERS     PP100_EnumPublishers = NULL;
	PP100_GETPUBLISHERSTATUS PP100_GetPublisherStatus = NULL;
	PP100_GETJOBSTATUS       PP100_GetJobStatus = NULL;
	PP100_GETJOBSTATUSEX     PP100_GetJobStatusEx = NULL;
	PP100_CANCELJOB          PP100_CancelJob = NULL;

public:
	// Constructor
	PP100API_POINTER() {}
	bool LoadPP100API();
	const char* last_dlerror(void) { return m_dlerror.c_str(); }
};


long endPP100Api();

extern PP100API_POINTER g_Api;

class TDMonitorEx : public QMainWindow, private Ui::TDMonitor
{
	Q_OBJECT

private:
	enum JobGroup {
		unfinished,
		finished
	};

	int colJobID     = 0;
	int colJobStatus = 1;
	int colJobPublisher = -1;

	int colUnfinishedCopies     = 2; // "Copies"
	int colUnfinishedJobType    = 3; // "Type"
	int colUnfinishedStackerIn  = 4; // "Input Stacker"
	int colUnfinishedStackerOut = 5; // "Output Stacker"
	int colUnfinishedRemaining  = 6; // "Time Needed"

	int colFinishedComletionNumber    = 2; // "Successful"
	int colFinishedErrorNumber        = 3; // "Failed"
	int colFinishedPublicationNumber  = 4; // "Quantity"
	int colFinishedJobType            = 5; // "Type"
	int colFinishedStackerOut         = 6; // "Output Stacker"

public:
	TDMonitorEx(
		const PP100_ENUM_PUBLISHER_INFO& sPublisherInfo,
		QMainWindow *parent = 0);
	~TDMonitorEx();

protected:
	std::deque<PP100_JOB_STATUS_EX> m_jobStatusUnfinished;
	std::deque<PP100_JOB_STATUS_EX> m_jobStatusFinished;
	bool m_bFirstGetPublisherStatusDone = false;
	long m_nRetGetPublisherStatus = PP100API_SUCCESS;
	PP100_PUBLISHER_STATUS m_publisherStatus = {};
	PP100_ENUM_PUBLISHER_INFO m_sPublisherInfo = {};
	std::unique_ptr<QTimer> m_timer;
	std::unique_ptr<QActionGroup> m_exclusivActionShowJobs;

	QMutex m_mtxFetchStatus;
	QMutex m_mtxUpdateScreen;

	void fetchStatus();
	void updateScreen_JobList();
	void updateScreen_DeviceStatus();
	void appendTreeItem(const PP100_JOB_STATUS_EX& jobStatus, JobGroup jobGroup);

	QString convJobType(const PP100_JOB_STATUS_EX& jobStatus);
	QString convJobStatus(const PP100_JOB_STATUS_EX& jobStatus);
	QString convStackerIn(const PP100_JOB_STATUS_EX& jobStatus);
	QString convStackerOut(const PP100_JOB_STATUS_EX& jobStatus);
	QString convRemainingEstimateTime(const PP100_JOB_STATUS_EX& jobStatus);

	void setDeviceInfo(const PP100_PUBLISHER_STATUS& publisherStatus, const std::deque<PP100_JOB_STATUS_EX>& jobStatusPublishing);
	void setDriveInfo(const PP100_PUBLISHER_STATUS& publisherStatus, int drvNum);
	void setStackerSetting(const PP100_PUBLISHER_STATUS& publisherStatus, int stckerNum);
	bool event(QEvent* event);

private slots:
	void cancelJob();
	void selectAllJobs();
	void showUnfinishedJobs();
	void showFinishedJobs();
	void showAboutDialog();
	void changeTab(int num);
	void updateStatusAsync();
	void updateScreen();
	void updateMenu();

signals:
	void updateScreenSignal();
};

long LoadPP100API(); // Load PP-100 Series API and get function pointers.

#endif // TDMONITOR_H
