#include <stdio.h>
#include <string.h>
#include <dlfcn.h>
#include <QApplication>
#include <QMainWindow>
#include <QDialog>
#include <QWidget>
#include <QtCore/QTranslator>
#include <QtCore/QLibraryInfo>
#include <QMessageBox>
#include <vector>

#include "TDMonitor.h"
#include "PublisherSelectDialog.h"

#include "config.h"

PP100API_POINTER g_Api;
std::vector<PP100_ENUM_PUBLISHER_INFO> g_arPublisherInfo;


void EnumPublisher(void)
{
	// Enumerate PP-100 Series.
	unsigned long ulPublisherInfoNum = 0;
	long nRet = g_Api.PP100_EnumPublishers(g_Api.ulHandle, NULL, &ulPublisherInfoNum);
	if(PP100API_MORE_ITEMS == nRet)
	{
		while(PP100API_MORE_ITEMS == nRet)
		{
			try {
				g_arPublisherInfo.resize(ulPublisherInfoNum);
			}
			catch(std::bad_alloc&)
			{
				printf("Allocating new memory failed.\n");
				g_arPublisherInfo.clear();
				return;
			}

			memset(&g_arPublisherInfo[0], 0, sizeof(PP100_ENUM_PUBLISHER_INFO) * g_arPublisherInfo.size());
			nRet = g_Api.PP100_EnumPublishers(g_Api.ulHandle, &g_arPublisherInfo[0], &ulPublisherInfoNum);
			if(PP100API_SUCCESS == nRet)
			{
				return;
			}
		}
	}
	else
	{
		printf("Enumerating to PC that TDBridge is running failed.\n");
	}
}


int main(int argc, char *argv[])
{
	QApplication app(argc, argv);

	// Qt Localization
	QTranslator qtTr;
	qtTr.load("qt_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
	app.installTranslator(&qtTr);

	QTranslator myTr;
	QString languageCode = QLocale::system().name();
	if ( myTr.load("TDMonitorQt_" + languageCode, INSTALL_DIR "/tr/") )
	{
		app.installTranslator(&myTr);
	}
	else
	{
		printf("ERROR: language file %s is not found\n", qPrintable(languageCode));
		printf("INSTALL_DIR = %s\n", INSTALL_DIR);
	}

	if( ! g_Api.LoadPP100API() )
	{
		(void) QMessageBox::critical(
			NULL,
			QApplication::translate("main function", "EPSON Total Disc Monitor"),
			QApplication::translate("main function", "Loading PP-100 Series API failed.") + g_Api.last_dlerror());
		return -1;
	}

	// Initialize PP100 Series API.
	long lRet = g_Api.PP100_Initialize();
	if(PP100API_SUCCESS != lRet)
	{
		printf("Initializing PP-100 Series API failed.\n");
		return lRet;
	}

	// Connect to PC that TDBridge is running.
	lRet = g_Api.PP100_ConnectServer(NULL, NULL, &g_Api.ulHandle);
	if(PP100API_SUCCESS != lRet)
	{
		printf("Connecting to PC that TDBridge is running failed.\n");
		return lRet;
	}

	PP100_ENUM_PUBLISHER_INFO sPublisherInfo = {};
	EnumPublisher();
	if (g_arPublisherInfo.empty())
	{
		(void) QMessageBox::warning(
			NULL,
			QApplication::translate("main function", "EPSON Total Disc Monitor"),
			QApplication::translate("main function", "There is no output device information so Total Disc Monitor cannot start."));
		return -2;
	}
	else if(argc > 1)
	{
		bool found = false;
		if(strlen(argv[1]) > 0)
		{
			printf("argv = %s\n", argv[1]);
			for(auto info : g_arPublisherInfo)
			{
				if (strcmp(info.pPublisherName, argv[1]) == 0)
				{
					memcpy(
						&sPublisherInfo,
						&info,
						sizeof(PP100_ENUM_PUBLISHER_INFO));
					found = true;
					break;
				}
			}
		}

		if(found == false)
		{
			printf("TDMonitor RegistID Error\n");
			endPP100Api();
			return -3;
		}
	}
	else
	{
		PublisherSelectDialog dialog(g_arPublisherInfo);
		dialog.show();
		if(dialog.exec() == QDialog::Accepted)
		{
			memcpy(
				&sPublisherInfo,
				&g_arPublisherInfo[ dialog.getSelectedPublisher() ],
				sizeof(PP100_ENUM_PUBLISHER_INFO));
		}
		else
		{
			printf("TDMonitor Cancel End\n");
			endPP100Api();
			return -4;
		}
	}

	TDMonitorEx dialog(sPublisherInfo);
	dialog.show();

	return app.exec();
}


// Load PP100API.so
bool PP100API_POINTER::LoadPP100API()
{
	const char *the_path = g_pPP100APIDLL;
	hPP100APIDLL = dlopen(the_path, RTLD_LAZY);
	if(NULL == hPP100APIDLL)
	{
		m_dlerror = dlerror();
		fprintf(stderr, "Loading PP-100 Series API failed. errno: %d\n%s\n", errno, m_dlerror.c_str());
		return false;
	}

	// Get function pointer of PP100_Initialize() and PP100_Destroy()
	PP100_Initialize         = (PP100_INITIALIZE)         dlsym(hPP100APIDLL, FNSTR_PP100_INITIALIZE);
	PP100_Destroy            = (PP100_DESTROY)            dlsym(hPP100APIDLL, FNSTR_PP100_DESTROY);
	PP100_ConnectServer      = (PP100_CONNECTSERVER)      dlsym(hPP100APIDLL, FNSTR_PP100_CONNECTSERVER);
	PP100_DisConnectServer   = (PP100_DISCONNECTSERVER)   dlsym(hPP100APIDLL, FNSTR_PP100_DISCONNECTSERVER);
	PP100_EnumPublishers     = (PP100_ENUMPUBLISHERS)     dlsym(hPP100APIDLL, FNSTR_PP100_ENUMPUBLISHERS);
	PP100_GetPublisherStatus = (PP100_GETPUBLISHERSTATUS) dlsym(hPP100APIDLL, FNSTR_PP100_GETPUBLISHERSTATUS);
	PP100_GetJobStatus       = (PP100_GETJOBSTATUS)       dlsym(hPP100APIDLL, FNSTR_PP100_GETJOBSTATUS);
	PP100_GetJobStatusEx     = (PP100_GETJOBSTATUSEX)     dlsym(hPP100APIDLL, FNSTR_PP100_GETJOBSTATUSEX);
	PP100_CancelJob          = (PP100_CANCELJOB)          dlsym(hPP100APIDLL, FNSTR_PP100_CANCELJOB);

	if(	NULL == PP100_Initialize         ||
		NULL == PP100_Destroy            ||
		NULL == PP100_ConnectServer      ||
		NULL == PP100_DisConnectServer   ||
		NULL == PP100_EnumPublishers     ||
		NULL == PP100_GetPublisherStatus ||
		(NULL == PP100_GetJobStatus && NULL == PP100_GetJobStatusEx) ||
		NULL == PP100_CancelJob  )
	{
		m_dlerror = dlerror();
		fprintf(stderr, "Getting function address failed. errno: %d\n%s\n", errno, m_dlerror.c_str());
		(void) dlclose(hPP100APIDLL);
		hPP100APIDLL = NULL;
		return false;
	}

	return true;
}
